import numpy as np
from aston.trace import Trace
from aston.peak.peak_models import gaussian
from aston.peak.peak_finding import (simple_peak_find, wavelet_peak_find,
                                     stat_slope_peak_find, event_peak_find)


def generate_chromatogram(n=5, twin=None):
    """
    Generates a trace with n gaussian peaks distributed through it.
    """
    if twin is None:
        twin = (0, 60)
    t = np.linspace(twin[0], twin[1], 300)
    peak_locs = twin[1] * np.random.random(n)
    peak_ws = 0.2 + 0.8 * np.random.random(n)
    peak_hs = 0.2 + 0.8 * np.random.random(n)

    y = np.zeros(len(t))
    for peak_loc, peak_w, peak_h in zip(peak_locs, peak_ws, peak_hs):
        y += gaussian(t, x=peak_loc, w=peak_w, h=peak_h)
    y += np.random.normal(scale=0.01, size=len(t))
    return Trace(y, t, ['X'])


def generate_gaussian():
    t = np.linspace(0, 60, 300)
    y = gaussian(t, x=30, w=2, h=1)
    return Trace(y, t, ['X'])


def test_simple_peak_find():
    trace = generate_gaussian()
    peaks = simple_peak_find(trace, init_slope=0.2, start_slope=0.2,
                             end_slope=0.2, min_peak_height=0.01)
    # TODO: better asserts
    assert len(peaks) > 0


def test_wavelet_peak_find():
    trace = generate_gaussian()
    peaks = wavelet_peak_find(trace)

    assert len(peaks) == 1
    peak = peaks[0]
    assert peak['h'] > 0.5 and peak['h'] < 1
    assert peak['t0'] < 30
    assert peak['t1'] > 30
    assert peak['x'] > 29.8 and peak['x'] < 30.2


def test_stat_slope_peak_find():
    trace = generate_gaussian()
    peaks = stat_slope_peak_find(trace)
    # FIXME & fix stat_slope?
    assert type(peaks) is list


def test_event_peak_find():
    trace = generate_gaussian()
    events = [{'t0': 25, 't1': 35}]

    # check passthrough
    peaks = event_peak_find(trace, events, adjust_times=False)
    assert peaks == events

    # check adjusted
    peaks = event_peak_find(trace, events, adjust_times=True)
    assert len(peaks) == 1
    assert peaks[0]['t0'] > 20 and peaks[0]['t0'] < 30
    assert peaks[0]['t1'] > 30 and peaks[0]['t1'] < 40



#from io import StringIO
import numpy as np
from aston.trace.trace import Chromatogram
from aston.tracefile.netcdf import write_netcdf


def test_write_netcdf():
    d = np.array([[0.1, 0.2, 0.1, 0.1],
                  [0.5, 0.6, 0.5, 0.4]]).T
    t = np.array([0, 1, 2, 3])
    df = Chromatogram(d, t, ['A', 'B'])
    #f_obj = StringIO()
    write_netcdf('test.cdf', df)



import unittest
import numpy as np
from aston.trace import Trace
from aston.peak import Peak, PeakComponent


class TestBoxPeak(unittest.TestCase):
    def setUp(self):
        # box peak
        info = {}
        trace = Trace(np.ones(10), np.arange(10), name=1)
        baseline = Trace([0, 9], [0, 0], name=1)
        c = PeakComponent(info, trace, baseline)
        self.peak = Peak('box', components=c)

    def test_area(self):
        assert self.peak.area() == self.peak.area(1)


class TestGaussianPeak(TestBoxPeak):
    def setUp(self):
        # box peak
        info = {}
        v = [0.043, 0.067, 0.094, 0.117, 0.131,
             0.131, 0.117, 0.094, 0.067, 0.043]
        trace = Trace(v, np.arange(10), name=1)
        baseline = Trace([0, 9], [0, 0], name=1)
        c = PeakComponent(info, trace, baseline)
        self.peak = Peak('gaussian', components=c)



from aston.spectra.isotopes import delta13C_Santrock


def test_delta13c():
    r45sam, r46sam = 1.1399413, 1.5258049
    d13cstd = -43.411
    r45std, r46std = 1.1331231, 1.4058630
    d13c = delta13C_Santrock(r45sam, r46sam, d13cstd, r45std, r46std,
                    ks='Isodat', d18ostd=-21.097)
    assert abs(d13c - (-40.30)) < 0.002



# FIXME
import numpy as np
from pyms.GCMS.Class import GCMS_data, Scan, IonChromatogram


class AstonPyMS(GCMS_data):
    """
    Adapter to allow pyms routines to run on aston files.

    Note: initialization uses Chromatogram, not scan lists
    as in pyms.


    Example
    -------
    from aston.tracefile.TraceFile import TraceFile
    from aston.trace.PyMS import AstonPyMS
    from pyms.GCMS.Function import build_intensity_matrix
    from pyms.Deconvolution.BillerBiemann.Function import BillerBiemann

    astonfile = TraceFile('~/Agilent.d/DATA.MS')
    raw_data = AstonPyMS(astonfile.data)

    a.info()

    im = build_intensity_matrix(raw_data)
    peaks = BillerBiemann(im)
    """

    def __init__(self, data):
        self.data = data

    def __len__(self):
        return self.data.shape[0]

    def get_min_mass(self):
        return min(self.data.columns)

    def get_max_mass(self):
        return max(self.data.columns)

    def get_index_at_time(self, time):
        time *= 60.0
        return np.argmin(np.abs(self.data.index - time))

    def get_time_list(self):
        return (self.data.index * 60.0).tolist()

    @property
    def scan_list(self):
        return self._scan_list

    @property
    def _scan_list(self):
        for scan in self.data.scans():
            yield Scan(scan.x.tolist(), scan.abn.tolist())

    def get_scan_list(self):
        return list(self._scan_list)

    def get_tic(self):
        return IonChromatogram(self.data.trace().values.T[0], \
                               (self.data.index * 60.0).tolist())

    def trim(self, begin=None, end=None):
        if begin is None and end is None:
            return

        if begin is None:
            st_idx = 0
        elif isinstance(begin, int):
            st_idx = begin
        else:
            st_idx = self.get_index_at_time(float(begin)) + 1

        if begin is None:
            en_idx = 0
        elif isinstance(end, int):
            en_idx = end
        else:
            st_idx = self.get_index_at_time(float(end)) + 1

        self.data = self.data[st_idx:en_idx]

    def info(self, print_scan_n=False):
        print(" Data retention time range: %.3f min -- %.3f min" % \
              (min(self.data.index), max(self.data.index)))
        tdiffs = np.diff(self.data.index)
        print(" Time step: %.3f s (std=%.3f s)" % \
              (np.mean(tdiffs), np.std(tdiffs)))
        print(" Number of scans: %d" % len(self))
        print(" Minimum m/z measured: %.3f" % self.get_min_mass())
        print(" Maximum m/z measured: %.3f" % self.get_max_mass())

        dfc = self.data.values.copy()
        dfc[dfc.nonzero()] = 1
        dfc = dfc.sum(axis=1)
        print(" Mean number of m/z values per scan: %d" % np.mean(dfc))
        print(" Median number of m/z values per scan: %d" % np.median(dfc))

    def write(self, file_root):
        f1name, f2name = file_root + '.I.csv', file_root + '.mz.csv'
        with open(f1name, 'w') as f1, open(f2name, 'w') as f2:
            for scan in self._scan_list:
                i_list = scan.get_intensity_list()
                f1.write(','.join('%.4f' % v for v in i_list))
                f1.write('\n')

                m_list = scan.get_mass_list()
                f2.write(','.join('%.4f' % v for v in m_list))
                f2.write('\n')

    def write_intensities_stream(self, file_name):
        with open(file_name, 'w') as f:
            for scan in self._scan_list:
                for i in scan.get_intensity_list():
                    f.write('%8.4f\n' % i)



#!/usr/bin/env python

#    Copyright 2011-2014 Roderick Bovee
#
#    This file is part of Aston.
#
#    Aston is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Aston is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Aston.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup
import matplotlib
import os

from aston import __version__


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

# read in the version number
with open('aston/__init__.py') as f:
    exec(f.read())

options = {
    'name': 'Aston',
    'version': __version__,
    'description': 'Mass/UV Spectral Analysis Program',
    'author': 'Roderick Bovee',
    'author_email': 'rbovee@gmail.com',
    'url': 'github.com/bovee/aston',
    'license': 'BSD 3-Clause',
    'platforms': ['Any'],
    'classifiers': [
        'Development Status :: 4 - Beta',
        'Environment :: X11 Applications :: Qt',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: Chemistry'
    ],
    'long_description': read('README.rst'),
    'packages': [
        'aston', 'aston.calibrations', 'aston.compat', 'aston.peak',
        'aston.spectra', 'aston.test', 'aston.trace', 'aston.tracefile'
    ],
    'scripts': [],
    'data_files': matplotlib.get_py2exe_datafiles(),
    'package_data': {'aston': []},
    'include_package_data': True,
    'install_requires': ['numpy', 'scipy', 'matplotlib'],
    'test_suite': 'nose.collector',
}

# all the magic happens right here
setup(**options)



import numpy as np
import base64
from aston.trace.trace import Trace, decompress


#TODO: reenable Trace merging someday, but with more intelligence?
#def test_and():
#    t = np.array([1, 2, 3, 4, 5])
#    a = Trace(np.array([10, 20, 30, 40, 50]), t, name='X')
#    b = Trace(np.array([11, 21, 31, 41, 51]), t, name='Y')
#    assert np.all(np.equal((a & b).values, np.array([10, 11])))


def test_add():
    t = np.array([1, 2, 3, 4, 5])
    a = Trace(np.array([10, 20, 30, 40, 50]), t, name='X')
    b = Trace(np.array([11, 21, 31, 41, 51]), t, name='X')
    c = a + b
    assert np.all(np.equal(c.values, np.array([21, 41, 61, 81, 101])))
    assert np.all(np.equal(c.index, np.array([1, 2, 3, 4, 5])))


def test_compress():
    a = Trace(np.array([10, 20, 30, 40, 50]), \
                    np.array([1, 2, 3, 4, 5]), name='X')
    zdata = 'eJxjZWBgEAHiaKUIpVgGhgZ7INsBiIC4' + \
            'AYgXADEIqEBpEyhtB6VdoLSnAwAZfgbw'
    assert base64.b64encode(a.compress()).decode('ascii') == zdata


def test_decompress():
    zdata = 'eJxjZWBgEAHiaKUIpVgGhgZ7INsBiIC4' + \
            'AYgXADEIqEBpEyhtB6VdoLSnAwAZfgbw'
    ts = decompress(base64.b64decode(zdata))
    assert ts.name == 'X'


#def test_tsfunc():
#    a = Trace(np.array([[0, 1, 2, 1, 0], [0, 0, 2, 0, 0]]).T, \
#                   np.array([1, 2, 3, 4, 5]), [1, 2])
#    grad = ts_func(np.gradient)
#    c = grad(a.trace(1)) / grad(Trace(a.times, a.times))
#    assert np.all(c.y == np.array([1., 1., 0., -1., -1.]))
